Class: Protocol::HTTP::ContentEncoding

Inherits:
Middleware show all
Defined in:
lib/protocol/http/content_encoding.rb

Overview

Encode a response according the the request’s acceptable encodings.

Constant Summary collapse

DEFAULT_WRAPPERS =
{
  'gzip' => Body::Deflate.method(:for)
}
DEFAULT_CONTENT_TYPES =
%r{^(text/.*?)|(.*?/json)|(.*?/javascript)$}

Constants inherited from Methods

Methods::CONNECT, Methods::DELETE, Methods::GET, Methods::HEAD, Methods::OPTIONS, Methods::PATCH, Methods::POST, Methods::PUT, Methods::TRACE

Instance Attribute Summary

Attributes inherited from Middleware

#delegate

Instance Method Summary collapse

Methods inherited from Middleware

build, #close, for

Methods inherited from Methods

each, valid?

Constructor Details

#initialize(app, content_types = DEFAULT_CONTENT_TYPES, wrappers = DEFAULT_WRAPPERS) ⇒ ContentEncoding

Returns a new instance of ContentEncoding.



21
22
23
24
25
26
# File 'lib/protocol/http/content_encoding.rb', line 21

def initialize(app, content_types = DEFAULT_CONTENT_TYPES, wrappers = DEFAULT_WRAPPERS)
  super(app)
  
  @content_types = content_types
  @wrappers = wrappers
end

Instance Method Details

#call(request) ⇒ Object



28
29
30
31
32
33
34
35
36
37
38
39
40
41
42
43
44
45
46
47
48
49
50
51
52
53
54
55
56
57
58
59
60
61
62
# File 'lib/protocol/http/content_encoding.rb', line 28

def call(request)
  response = super
  
  # Early exit if the response has already specified a content-encoding.
  return response if response.headers['content-encoding']
  
  # This is a very tricky issue, so we avoid it entirely.
  # https://lists.w3.org/Archives/Public/ietf-http-wg/2014JanMar/1179.html
  return response if response.partial?
  
  # Ensure that caches are aware we are varying the response based on the accept-encoding request header:
  response.headers.add('vary', 'accept-encoding')
  
  # TODO use http-accept and sort by priority
  if !response.body.empty? and accept_encoding = request.headers['accept-encoding']
    
    if content_type = response.headers['content-type'] and @content_types =~ content_type
      body = response.body
      
      accept_encoding.each do |name|
        if wrapper = @wrappers[name]
          response.headers['content-encoding'] = name
          
          body = wrapper.call(body)
          
          break
        end
      end
      
      response.body = body
    end
  end
  
  return response
end