Exception: Protocol::HTTP::RefusedError

Inherits:
Error
  • Object
show all
Defined in:
lib/protocol/http/error.rb

Overview

Raised when an HTTP stream or request was refused before any processing occurred. In the case of requests, it indicates that the request was refused before any processing occurred, and can be safely retried.