Exception: Protocol::HTTP::RemoteError

Inherits:
Error
  • Object
show all
Defined in:
lib/protocol/http/error.rb

Overview

Raised when the remote endpoint fails while processing an HTTP stream or request. This error does not indicate whether application processing occurred. Where available, the protocol-specific error should be attached as the cause.