Class: Protocol::HTTP::Header::Cookie
- Defined in:
- lib/protocol/http/header/cookie.rb
Overview
The cookie header contains stored HTTP cookies previously sent by the server with the set-cookie header.
It is used by clients to send key-value pairs representing stored cookies back to the server.
Direct Known Subclasses
Instance Method Summary collapse
-
#to_h ⇒ Object
Parses the
cookieheader into a hash of cookie names and their corresponding cookie objects.
Methods inherited from Multiple
Constructor Details
This class inherits a constructor from Protocol::HTTP::Header::Multiple