Class: Protocol::HTTP::Header::SetCookie
- Defined in:
- lib/protocol/http/header/set_cookie.rb
Overview
The set-cookie header sends cookies from the server to the user agent.
Each Set-Cookie header must be a separate header field — they cannot be combined.
It is used to store cookies on the client side, which are then sent back to the server
in subsequent requests using the cookie header.
Class Method Summary collapse
-
.trailer? ⇒ Boolean
Whether this header is acceptable in HTTP trailers.
Instance Method Summary collapse
-
#to_h ⇒ Object
Parses the
set-cookieheaders into a hash of cookie names and their corresponding cookie objects.
Methods inherited from Multiple
coerce, #initialize, parse, #to_s
Constructor Details
This class inherits a constructor from Protocol::HTTP::Header::Multiple
Class Method Details
.trailer? ⇒ Boolean
Whether this header is acceptable in HTTP trailers.
31 32 33 |
# File 'lib/protocol/http/header/set_cookie.rb', line 31 def self.trailer? false end |