Class: Protocol::HTTP::Headers::Merged

Inherits:
Object
  • Object
show all
Includes:
Enumerable
Defined in:
lib/protocol/http/headers.rb

Overview

Used for merging objects into a sequential list of headers. Normalizes header keys and values.

Instance Method Summary collapse

Constructor Details

#initialize(*all) ⇒ Merged

Construct a merged list of headers.



513
514
515
# File 'lib/protocol/http/headers.rb', line 513

def initialize(*all)
	@all = all
end

Instance Method Details

#<<(headers) ⇒ Object

Add a new set of headers to the merged list.



535
536
537
538
539
# File 'lib/protocol/http/headers.rb', line 535

def << headers
	@all << headers
	
	return self
end

#clearObject

Clear the references to all headers.



528
529
530
# File 'lib/protocol/http/headers.rb', line 528

def clear
	@all.clear
end

#each(&block) ⇒ Object

Enumerate all headers in the merged list.



546
547
548
549
550
551
552
553
554
# File 'lib/protocol/http/headers.rb', line 546

def each(&block)
	return to_enum unless block_given?
	
	@all.each do |headers|
		headers.each do |key, value|
			yield key.to_s.downcase, value.to_s
		end
	end
end

#fieldsObject



518
519
520
# File 'lib/protocol/http/headers.rb', line 518

def fields
	each.to_a
end

#flattenObject



523
524
525
# File 'lib/protocol/http/headers.rb', line 523

def flatten
	Headers.new(fields)
end